#include <os.h>
#include <SDL/SDL.h>
#include "nc.h"

SDL_Surface *screen;
SDL_nFont *font_b, *font_w;
SDL_bool done = SDL_FALSE;
char filenames[128][BUF_SIZE];
int num_files;
int file_choice = 0;
int file_scroll = 0;

void init(void) {
	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
		printf("Couldn't initialize SDL: %s\n", SDL_GetError());
		exit(EXIT_FAILURE);
	}
	screen = SDL_SetVideoMode(320, 240, is_cx ? 16 : 8, SDL_SWSURFACE);
	if (screen == NULL) {
		printf("Couldn't set video mode: %s\n", SDL_GetError());
		SDL_Quit();
		exit(EXIT_FAILURE);
	}
	font_b = SDL_nLoadFont(NSP_FONT_TINYTYPE, SDL_MapRGB(screen->format, 0, 0, 0), NSP_FONTCFG_DEFAULT);
	if (font_b == NULL) {
		printf("Couldn't load font\n");
		SDL_Quit();
		exit(EXIT_FAILURE);
	}
	font_w = SDL_nLoadFont(NSP_FONT_TINYTYPE, SDL_MapRGB(screen->format, 255, 255, 255), NSP_FONTCFG_DEFAULT);
	if (font_w == NULL) {
		printf("Couldn't load font\n");
		SDL_nFreeFont(font_b);
		SDL_Quit();
		exit(EXIT_FAILURE);
	}
	SDL_EnableKeyRepeat(500, 200);
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));
}

void quit(void) {
	SDL_nFreeFont(font_b);
	SDL_nFreeFont(font_w);
	SDL_Quit();
}

#define DRAW_PANEL_RECT() SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0, 0, 0))
#define DRAW_STRING_BOLD(font, x, y, s) do { \
	SDL_nDrawString(screen, font, x, y, s); \
	SDL_nDrawString(screen, font, x + 1, y, s); \
} while (0)

void draw_panel(void) {
	char current_dir[BUF_SIZE];
	SDL_Rect rect;
	rect.x = rect.y = 0;
	rect.w = 320;
	rect.h = 24;
	DRAW_RECT();
	rect.y = 216;
	DRAW_RECT();
	rect.x = rect.y = 0;
	rect.w = 1;
	rect.h = 240;
	DRAW_RECT();
	rect.x = 160;
	DRAW_RECT();
	rect.x = 319;
	DRAW_RECT();
	rect.x = 0;
	rect.y = 239;
	rect.w = 320;
	rect.h = 1;
	DRAW_RECT();
	rect.x = 160;
	rect.y = 72;
	rect.w = 160;
	DRAW_RECT();
	SDL_nDrawString(screen, font_w, 8, 8, "Ndless Commander " VERSION);
	DRAW_STRING_BOLD(font_b, 24, 40, "Name");
	NU_Current_Dir("A:", current_dir);
	SDL_nDrawString(screen, font_w, 8, 224, current_dir);
}

void draw_filenames(void) {
	SDL_Rect rect;
	int i;
	if(num_files <= NUM_ITEMS_SHOWN)
		file_scroll = 0;
	rect.x = 24;
	rect.y = 56;
	rect.w = 136;
	rect.h = 8;
	get_filenames();
	for (i = file_scroll; i < num_files && i < file_scroll + NUM_ITEMS_SHOWN; ++i) {
		SDL_nDrawStringInRect(screen, font_b, &rect, filenames[i]);
		if (strcmp(filenames[i], ".") != 0
		 && strcmp(filenames[i], "..") != 0
		 && is_directory(filenames[i]))
			SDL_nDrawString(screen, font_b, 16, rect.y, "\\");
		rect.y += 8;
	}
	SDL_nDrawString(screen, font_b, 8, 56 + (8 * (file_choice - file_scroll)), "\x10");
	if (file_scroll > 0)
		SDL_nDrawString(screen, font_b, 8, 48, "\x1e");
	if (file_scroll + NUM_ITEMS_SHOWN < num_files)
		SDL_nDrawString(screen, font_b, 8, 200, "\x1f");
}

void draw_file_info(void) {
	struct stat file_stat;
	DRAW_STRING_BOLD(font_b, 176, 40, "File:");
	SDL_nDrawString(screen, font_b, 211, 40, filenames[file_choice]);
	DRAW_STRING_BOLD(font_b, 176, 48, "Size:");
	if (strcmp(filenames[file_choice], ".") != 0
	 && strcmp(filenames[file_choice], "..") != 0
	 && !is_directory(filenames[file_choice])) {
	 	FILE *fp;
		char buffer[BUF_SIZE] = {'\0'};
		stat(filenames[file_choice], &file_stat);
		get_readable_size(file_stat.st_size, buffer);
		SDL_nDrawString(screen, font_b, 211, 48, "%s", buffer);
		fp = fopen(filenames[file_choice], "r");
		if (fp) {
			char file_content[BUF_SIZE] = {'\0'};
			SDL_Rect rect = {164, 76, 152, 137};
			int c, i = 0;
			do {
				c = fgetc(fp);
				file_content[i] = c;
				++i;
			} while (c != EOF && i + 1 < BUF_SIZE);
			SDL_nDrawStringInRect(screen, font_b, &rect, file_content);
			fclose(fp);
		}
	} else {
		SDL_nDrawString(screen, font_b, 168, 140, "No preview available \x2");
		SDL_nDrawString(screen, font_b, 211, 48, "N/A");
	}
}

char *get_readable_size(int size, char *buffer) {
    const char units[4][3] = {"B\0", "kB\0", "MB\0", "GB\0"};
    int i = 0;
    while (size >= 1024) {
        size /= 1024;
        ++i;
    }
    sprintf(buffer, "%d%s", size + 1, units[i]);
    return buffer;
}

int is_directory(const char *path) {
	struct stat file_stat;
	stat(path, &file_stat);
	return file_stat.st_mode & S_IFDIR;
}

void get_filenames(void) {
	struct dstat file_info;
	int i = num_files = 0;
	NU_Get_First(&file_info, "*.*");
	do {
		strncpy(filenames[i++], file_info.filepath, BUF_SIZE);
		++num_files;
	} while (NU_Get_Next(&file_info) == 0);
	NU_Done(&file_info);
}

void handle_return_key(void) {
	if (is_directory(filenames[file_choice])) {
		NU_Set_Current_Dir(filenames[file_choice]);
		get_filenames();
		file_choice = file_scroll = 0;
	}
}

void handle_keydown(SDLKey key) {
	switch (key) {
		case SDLK_UP:
			--file_choice;
			if (file_choice < 0) {
				file_choice = num_files - 1;
				file_scroll = num_files - NUM_ITEMS_SHOWN;
			} else if (file_choice - file_scroll < 0)
				--file_scroll;
			break;
		case SDLK_DOWN:
			++file_choice;
			if (file_choice >= num_files)
				file_choice = file_scroll = 0;
			else if (file_choice - file_scroll >= NUM_ITEMS_SHOWN)
				++file_scroll;
			break;
		case SDLK_RETURN:
			handle_return_key();
			break;
		case SDLK_ESCAPE:
			done = SDL_TRUE;
			break;
		default:
			break;
	}
}

int main(void) {
	init();
	while (!done) {
		SDL_Event event;
		SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));
		draw_panel();
		draw_filenames();
		draw_file_info();
		SDL_Flip(screen);
		SDL_WaitEvent(&event);
		if (event.type == SDL_KEYDOWN)
			handle_keydown(event.key.keysym.sym);
	}
	quit();
	return EXIT_SUCCESS;
}